/**
 * Copyright (c) Members of the EMI Collaboration. 2011.
 * See http://eu-emi.eu/partners/ for details on the copyright holders.
 * For license conditions see http://www.apache.org/licenses/LICENSE-2.0
 */
package org.glite.pseudo.server.aaclient.impl;

import java.security.cert.X509Certificate;
import java.util.Properties;

import org.apache.axis.configuration.SimpleProvider;
import org.glite.pseudo.server.PseudoServerException;
import org.glite.pseudo.server.aaclient.AAClient;
import org.glite.pseudo.server.aaclient.AAClientException;
import org.glite.pseudo.server.caclient.impl.AbstractSSLClient;
import org.glite.pseudo.server.config.PseudoServerConfiguration;
import org.glite.security.trustmanager.ContextWrapper;
import org.glite.security.trustmanager.axis.SSLConfigSender;
import org.glite.security.util.DN;
import org.glite.security.util.DNHandler;
import org.glite.security.voms.service.certificates.VOMSCertificates;
import org.glite.security.voms.service.certificates.VOMSCertificatesServiceLocator;
import org.ini4j.Ini;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * VOMSClient registers the certificates to the VOMS server using VOMS-Admin Web
 * service interface
 * </p>
 */
public class VOMSClient extends AbstractSSLClient implements AAClient {

    /** Logging */
    private static final Logger log = LoggerFactory.getLogger(VOMSClient.class);

    /** VOMS Certificate Service */
    private VOMSCertificates vomsCertService;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.glite.pseudo.server.aaclient.AAClient#registerCertificate(java.security
     * .cert.X509Certificate, java.security.cert.X509Certificate)
     */
    public void registerCertificate(X509Certificate original,
            X509Certificate alias) throws AAClientException {

        DN originalDN = DNHandler.getDNRFC2253(original.getSubjectDN()
                .toString());
        DN caDN = DNHandler.getDNRFC2253(original.getIssuerDN().toString());
        DN aliasDN = DNHandler.getDNRFC2253(alias.getSubjectDN().toString());

        log.info("Registering " + aliasDN + " as an alias to " + originalDN);

        try {
            log.debug("Generating VOMS X509Certificate");
            org.glite.security.voms.service.certificates.X509Certificate newCert = new org.glite.security.voms.service.certificates.X509Certificate(
                    alias.getEncoded(), alias.getSerialNumber().longValue(),
                    alias.getIssuerDN().toString(), alias.getNotAfter()
                            .toString(), aliasDN.toString());
            this.vomsCertService.addCertificate(originalDN.getX500(),
                    caDN.getX500(), newCert);
        } catch (Exception e) {
            throw new AAClientException(
                    "Could not add new certificate to the VOMS certificate service!",
                    e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.glite.pseudo.server.PseudoServerComponent#init(org.glite.pseudo.server
     * .config.PseudoServerConfiguration)
     */
    public void init(PseudoServerConfiguration configuration)
            throws PseudoServerException {
        log.debug("Initializing VOMSClient");
        Ini.Section cfgSection = configuration.getAAClientConfiguration();
        super.init(cfgSection);
        Properties properties = System.getProperties();
        if (keyFile == null) {
            throw new PseudoServerException(
                    "Private key for client authentication is not configured!");
        } else {
            log.debug("Using {} as a private key for client authentication.",
                    keyFile);
            properties.setProperty("sslKey", keyFile);
        }

        if (certFile == null) {
            throw new PseudoServerException(
                    "Client authentication certificate is not configured!");
        } else {
            log.debug("Using {} as a certificate for client authentication.",
                    certFile);
            properties.setProperty("sslCertFile", certFile);
        }

        if (keyPasswd == null) {
            log.debug("Password for the private key is not configured, the private key should be plaintext.");
        } else {
            log.debug("Using a password for decrypting the private key.");
            properties.setProperty("sslKeyPasswd", keyPasswd);
        }

        if (crlCheck != null && crlCheck.toLowerCase().equals("false")) {
            log.debug("CRL Check is NOT required.");
            properties.setProperty(ContextWrapper.CRL_REQUIRED, "false");
        } else {
            properties.setProperty(ContextWrapper.CRL_REQUIRED, "true");
        }

        try {
            SimpleProvider provider = SSLConfigSender
                    .getTransportProvider(properties);
            VOMSCertificatesServiceLocator locator = new VOMSCertificatesServiceLocator(
                    provider);
            locator.setVOMSCertificatesEndpointAddress(endpoint);
            this.vomsCertService = locator.getVOMSCertificates();
        } catch (Exception e) {
            throw new PseudoServerException(
                    "Error during VOMS certificate service client initialization!",
                    e);
        }

        log.info("VOMS certificate service client successfully initialized.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.glite.pseudo.server.caclient.impl.AbstractSSLClient#shutdown()
     */
    public void shutdown() {
        super.shutdown();

    }

}
